!----------------------------------------------------------------------
!----------------------------------------------------------------------
! Copyright (C) 2004, 2005 Chaok Seok, Evangelos Coutsias and Ken Dill
!      UCSF, Univeristy of New Mexico, Seoul National University
! Witten by Chaok Seok and Evangelos Coutsias 2004.
                                                                                
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
                                                                                
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
                                                                                
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA!----------------------------------------------------------------------------
!-----------------------------------------------------------------------
!-----------------------------------------------------------------------
MODULE in_out
!-----------------------------------------------------------------------
! subroutines reading and writing pdb files
!-----------------------------------------------------------------------

  use ls_rmsd
  implicit none
  integer, parameter :: len_fname = 30 ! max length of a file name
  integer, parameter :: len_s = 30 ! max length of a short string
  integer, parameter :: len_l = 80 ! max length of a long string

!-----------------------------------------------------------------------
CONTAINS
!-----------------------------------------------------------------------
subroutine get_num_atm(rmsd_type, pdb, num_atm, ios)
!-----------------------------------------------------------------------
! gets num of atm from a pdb file
!-----------------------------------------------------------------------
  character(len=2), intent(in) :: rmsd_type
  character(len=len_fname), intent(in) :: pdb
  integer, intent(out) :: num_atm, ios
  character(len=3) :: atmname
  character(len=len_s) :: string
  integer :: unit=10, i, ioerror
  !-----------------------------------------------------------------------
  open(unit, file = trim(pdb), iostat = ios, status = 'old')
  if (ios /= 0) then
     return
  end if
  do    !endless loop, exit at ATOM
     read(unit, 50) string
     if (string == 'ATOM') exit
  end do
  backspace(unit)
  num_atm = 0
  do    
     read(unit, 50,iostat=ioerror) string
     if (ioerror < 0) exit ! end of file
     if (string /= 'ATOM' .and. string /= 'ANISOU' .and. string /= 'HETATM'& 
          .and. string /= 'SIGATM' .and. string /= 'SIGUIJ') exit
     if (string == 'ATOM' .or. string == 'HETATM') then
        backspace(unit)
        read(unit,87) atmname
87      format (13x,a3)
        if (string/='ATOM') exit
        if ((rmsd_type=='ca'.and.atmname=='CA') .or. &
             (rmsd_type=='bb'.and. &
             (atmname=='CA'.or.atmname=='N'.or.atmname=='C')).or.&
             (rmsd_type=='aa')) then
           num_atm=num_atm+1
        end if
     end if
  end do
  close(unit)

50 format (a6)
65 format (a26)

end subroutine get_num_atm
!-----------------------------------------------------------------------
subroutine read_pdb(rmsd_type, pdb, coord, N)
!-----------------------------------------------------------------------
! reads a pdb file and gets coordinates.
!-----------------------------------------------------------------------
  character(len=2), intent(in) :: rmsd_type
  character(len=len_fname), intent(in) :: pdb
  real(dp), dimension(:,:), intent(out) :: coord
  integer, intent(out) :: N
  character(len=10) :: atmname, res_name, cur_res, string
  integer :: unit=10, atm_no, i, ioerror
  real(dp) :: x, y, z
!-----------------------------------------------------------------------
  open(unit, file = trim(pdb))
  do    !endless loop, exit at ATOM
     read(unit, 50) string
     if (string == 'ATOM') exit
  end do
  backspace(unit)
  atm_no = 0
  do    !endless loop, exit if not ATOM
     read(unit, 50,iostat=ioerror) string
     if (ioerror < 0) exit ! end of file
     if (string /= 'ATOM' .and. string /= 'ANISOU' .and. string /= 'HETATM'& 
          .and. string /= 'SIGATM' .and. string /= 'SIGUIJ') exit
     if (string == 'ATOM' .or. string == 'HETATM') then
        backspace(unit)
        read(unit,87) atmname, res_name, cur_res, x, y, z
87      format (13x,a3,1x,a3,2x,a4,4x,3f8.3)
        if ((rmsd_type=='ca'.and.atmname=='CA') .or. &
             (rmsd_type=='bb'.and. &
             (atmname=='CA'.or.atmname=='N'.or.atmname=='C')).or.&
             (rmsd_type=='aa')) then
           atm_no=atm_no+1
           coord(1:3,atm_no) = (/ x, y, z /)
        end if
     end if
  end do
  N = atm_no
  close(unit)
50 format (a6)
65 format (a26)

end subroutine read_pdb
!-----------------------------------------------------------------------
subroutine read_bb(pdb, num_res, residue, coord)
!-----------------------------------------------------------------------
! reads a pdb file and gets coordinates.
!-----------------------------------------------------------------------
  character(len=len_fname), intent(in) :: pdb
  real(dp), dimension(:,:), intent(out) :: coord
  integer, intent(out) :: num_res
  character(len=3), intent(out) :: residue(:)
  character(len=10) :: atmname, res_name, string, prev_res, cur_res
  integer :: unit=12, i, ioerror, res_no, idx
  real(dp) :: x, y, z
  integer, parameter :: max_res = 1000
  logical :: atom_read(3,max_res)
!-----------------------------------------------------------------------

  atom_read(:,:) = .false.

  open(unit, file = trim(pdb))
  do    
     read(unit, 50) string
     if (string == 'ATOM') exit
  end do

  backspace(unit)
  res_no = 0
  prev_res = '-1'
  do    
     read(unit, 50,iostat=ioerror) string
     if (ioerror < 0) exit ! end of file
     if (string /= 'ATOM' .and. string /= 'ANISOU' .and. string /= 'HETATM'& 
          .and. string /= 'SIGATM' .and. string /= 'SIGUIJ') exit
     if (string == 'ATOM' .or. string == 'HETATM') then
        backspace(unit)
        read(unit,87) atmname, res_name, cur_res, x, y, z
!        print*, atmname, res_name, cur_res
87      format (13x,a3,1x,a3,2x,a4,4x,3f8.3)
        if (cur_res /= prev_res) then
           res_no = res_no + 1
           prev_res = cur_res
           residue(res_no) = res_name
           ! Heuristic way of excluding cap
           if (cur_res == 'ACE') then
              res_no = res_no - 1
           end if
        end if
        if (atmname=='N') then
           idx = 1
        else if (atmname=='CA') then
           idx = 2
        else if (atmname=='C') then
           idx = 3
        else
           idx = 0
        end if
        if (idx > 0) then
           atom_read(idx,res_no) = .true.
           coord(1:3, 3*(res_no - 1) + idx) = (/ x, y, z /)
        end if
     end if
  end do
  close(unit)
  num_res = res_no

  do res_no = 1, num_res
     do idx = 1, 3
        if (.not.atom_read(idx,res_no) .and. residue(res_no) /= 'ACE') then
           print*, 'Error in reading pdb file ', trim(pdb)
           print*, res_no, idx
!           stop
        end if
     end do
  end do
50 format (a6)
65 format (a26)

end subroutine read_bb
!-----------------------------------------------------------------------
subroutine write_pdb(pdb_in, pdb_out, U, center, center0)
!-----------------------------------------------------------------------
  character(len=len_fname), intent(in) :: pdb_in, pdb_out
  real(dp), dimension(3,3), intent(in) :: U 
  real(dp), dimension(3), intent(in) :: center, center0
  character(len=10) :: atmname, res_name, string
  integer :: unit_in=20, unit_out=21, ios, i, ioerror, res_no
  real(dp), dimension(3) :: R, R_rot

  open(unit_in, file = trim(pdb_in))
  open(unit_out, file = trim(pdb_out))
  do    !endless loop, exit at ATOM
     read(unit_in, 50) string
     if(string == 'ATOM') exit
  end do
  backspace(unit_in)

  do    !endless loop, exit if not ATOM
     read(unit_in, 50,iostat=ioerror) string
     if (ioerror < 0) exit
     if (string /= 'ATOM' .and. string /= 'ANISOU' .and. string /= 'HETATM'& 
          .and. string /= 'SIGATM' .and. string /= 'SIGUIJ') exit
     if (string == 'ATOM' .or. string == 'HETATM') then
        read(unit_in,87) atmname, res_name, res_no, R(1:3)
87      format (13x,a3,1x,a3,2x,i4,4x,3f8.3)
        R_rot = matmul(U, R - center) + center0 ! translate and rotate
        write(unit_out,88) string, atmname, res_name, res_no, R_rot(1:3)
88      format (a6,7x,a3,1x,a3,2x,i4,4x,3f8.3)
     end if
  end do
  write(unit_out,50) string
  close (unit_in)
  close (unit_out)
50 format (a6)
65 format (a26)
60 format (a26, 4x, 3f8.3)

end subroutine write_pdb
!-----------------------------------------------------------------------
subroutine write_bb(pdb, num_res, residue, coord, U, center, center0)
!-----------------------------------------------------------------------
  character(len=len_fname), intent(in) :: pdb
  real(dp), dimension(:,:), intent(in) :: coord
  real(dp), dimension(3,3), intent(in) :: U 
  real(dp), dimension(3), intent(in) :: center, center0
  integer, intent(in) :: num_res
  character(len=3), intent(in) :: residue(:)
  integer :: unit=21, atm_no, res_no, idx

  open(unit, file = trim(pdb))
  atm_no = 0
  do res_no = 1, num_res
     idx = 3*(res_no-1)
     atm_no = atm_no + 1
     write(unit, 87) atm_no, 'N  ', residue(res_no), res_no, & 
          matmul(U, coord(1:3,idx+1) - center) + center0
     atm_no = atm_no + 1
     write(unit, 87) atm_no, 'CA ', residue(res_no), res_no, & 
          matmul(U, coord(1:3,idx+2) - center) + center0
     atm_no = atm_no + 1
     write(unit, 87) atm_no, 'C  ', residue(res_no), res_no, & 
          matmul(U, coord(1:3,idx+3) - center) + center0
  end do
  write(unit,50) 
  close(unit)
50 format ('END')
87 format ('ATOM',2x,i5,2x,a3,1x,a3,2x,i4,4x,3f8.3)

end subroutine write_bb
!-----------------------------------------------------------------------
end MODULE in_out
!-----------------------------------------------------------------------
