!----------------------------------------------------------------------
!----------------------------------------------------------------------
! Copyright (C) 2004, 2005 Chaok Seok, Evangelos Coutsias and Ken Dill
!      UCSF, Univeristy of New Mexico, Seoul National University
! Witten by Chaok Seok and Evangelos Coutsias 2004.
                                                                                
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!
                                                                                
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!
                                                                                
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA!----------------------------------------------------------------------------
!-----------------------------------------------------------------------
program main_opt 
!-----------------------------------------------------------------------
! Read a list of pdb files, and generate a backbone structure which takes
! canonical bond angles and omega angle that best fit the original 
! structure for each pdb by optimizing phi/psi angles.
!-----------------------------------------------------------------------
  use ls_rmsd
  use in_out
  use geometry
!-----------------------------------------------------------------------
  implicit none
  character(len=len_fname) :: pdb_list, in_pdb, out_pdb
  real(dp) :: coord_init(3,3*max_res), coord_opt(3,3*max_res)
  integer :: ios, num_res, unit_list = 11, n_dof
  character(len=3) :: residue(max_res)
  real(dp) :: U(3,3), center1(3), center2(3), error_init, error_opt
!-----------------------------------------------------------------------
  logical :: PRO_omg_test = .false.
  real(dp) :: r_a1(3), r_c(3), r_n(3), r_a2(3), r(3,3), omg
  integer :: res_no, domg

  ! read pdb file names and rmsd type
  print*, 'pdb file list?'
  read*, pdb_list
  print*, trim(pdb_list)
  print*, ''

  open(unit_list, file = trim(pdb_list))
  do

     read(unit_list, "(a)", iostat = ios) in_pdb
     if (ios < 0) exit

     print*, 'reading ', trim(in_pdb)
     call read_bb(in_pdb, num_res, residue, coord_init)
     if (PRO_omg_test) then
        do res_no = 1, num_res
           if (residue(res_no) == 'PRO' .and. res_no < num_res) then
              r_a1(:) = coord_init(:, 3*res_no-1)
              r_c(:) = coord_init(:, 3*res_no)
              r_n(:) = coord_init(:, 3*res_no+1)
              r_a2(:) = coord_init(:, 3*res_no+2)
              r(:,1) = r_c(:) - r_a1(:)
              r(:,2) = r_n(:) - r_c(:)
              r(:,3) = r_a2(:) - r_n(:)
              call calc_angle(1, r, omg)
              do while (omg > pi) 
                 omg = omg - 2.0d0*pi
              end do
              do while (omg < -pi) 
                 omg = omg + 2.0d0*pi
              end do
              if (abs(omg) < 0.5d0*pi) then
                 print*, trim(in_pdb), num_res
                 print*, 'PRO omega', res_no, omg*rad2deg
              end if
           end if
        end do
     end if

     print*, 'num_res= ', num_res

     n_dof = 2*num_res - 2
!     call test_drmsd_dw(num_res, coord_opt, coord_init)
     call optimize_phi_psi(n_dof, num_res, coord_init, coord_opt, & 
          error_init, error_opt, U, center2, center1)
     print*, 'Init and opt error=', error_init, error_opt
     call f77flush()

     out_pdb = trim(in_pdb)//"_opt"
     print*, 'writing: ', trim(out_pdb)
     call write_bb(out_pdb, num_res, residue, coord_opt, U, center2, center1) 
     print*, ''

  end do
  close(unit_list)

222 format('-----------------------------------------------------------')

!-----------------------------------------------------------------------
end program main_opt
!-----------------------------------------------------------------------

