!----------------------------------------------------------------------
! Copyright (C) 2004, 2005 Chaok Seok, Evangelos Coutsias and Ken Dill
!      UCSF, Univeristy of New Mexico, Seoul National University
! Witten by Chaok Seok and Evangelos Coutsias 2004.  
                                                                                
! This library is free software; you can redistribute it and/or
! modify it under the terms of the GNU Lesser General Public
! License as published by the Free Software Foundation; either
! version 2.1 of the License, or (at your option) any later version.
!                                                                                 
! This library is distributed in the hope that it will be useful,
! but WITHOUT ANY WARRANTY; without even the implied warranty of
! MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
! Lesser General Public License for more details.
!                                                                                 
! You should have received a copy of the GNU Lesser General Public
! License along with this library; if not, write to the Free Software
! Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
!----------------------------------------------------------------------------
!-----------------------------------------------------------------------
program main_rmsd 
!-----------------------------------------------------------------------
! reads two pdb files and calculate rmsd (C-alpha, bakcbone, or all-atom)
!-----------------------------------------------------------------------
  use in_out
  use ls_rmsd
!-----------------------------------------------------------------------
  implicit none
  character(len=len_fname), dimension(2) :: pdb
  character(len=len_fname) :: out_pdb
  character(len=2) :: rmsd_type
  logical :: extra_out
  integer, dimension(2) :: num_atm
  integer :: i, j, N, ios, ioerror
  real(dp), dimension(:,:), allocatable :: coord1, coord2
  real(dp), dimension(3,3) :: U 
  real(dp), dimension(3) :: center1, center2
  real(dp) :: error
  character(len=10) :: string
  logical :: calc_g = .false.
  real(dp), allocatable :: g(:,:)
!-----------------------------------------------------------------------

  ! read pdb file names and rmsd type
  print*, 'first pdb file name?'
  read*, pdb(1)
  print*, trim(pdb(1))
  print*, 'second pdb file name?'
  read*, pdb(2)
  print*, trim(pdb(2))
  print*, 'rmsd type? (ca for CA, bb for backbone, or aa for all atom)'
  read*, rmsd_type
  print*, rmsd_type
  print*, 'output superposed pdb (T or F)?'
  read*, extra_out
  if (extra_out) then
     print*, 'pdb name for superposed structure?'
     read*, out_pdb
  end if

  ! get number of atoms
  do j = 1, 2
     call get_num_atm(rmsd_type, pdb(j), num_atm(j), ios)
     if (ios /= 0) then
        write(*,*) 'Error during opening of ', trim(pdb(j))
        stop
     end if
  end do

  if (num_atm(1)/=num_atm(2)) then
     write(*,*)'Error:'
     write(*,*)'Number of atoms in ',trim(pdb(1)),' and ',trim(pdb(2)),&
          ' are different.', num_atm(1:2)
     stop
  end if

  ! read coordinates
  N = num_atm(1)
  allocate(coord1(3,N), coord2(3,N))
  call read_pdb(rmsd_type, pdb(1), coord1, N)
  call read_pdb(rmsd_type, pdb(2), coord2, N)

  ! calculate rmsd
  allocate(g(2,num_atm(1)))
  call rmsd(N, coord2, coord1, 1, U, center2, center1, error, calc_g, g)

  print*, 'rmsd = ', error

  if (extra_out) then
     print*, 'writing ', trim(out_pdb)
     call write_pdb (pdb(2), out_pdb, U, center2, center1) 
  end if

  deallocate(coord1, coord2)

  write(*,222)
222 format('-----------------------------------------------------------')

!-----------------------------------------------------------------------
end program main_rmsd
!-----------------------------------------------------------------------

